package Mixi;

use strict;
use warnings;
use Carp;

use LWP::UserAgent;
use HTTP::Cookies;
use HTTP::Headers;
use HTTP::Request;

use constant DOMAIN_NAME      => 'mixi.jp';
use constant BASE_URL         => sprintf("http://%s/", DOMAIN_NAME);
use constant HOME_URL         => BASE_URL . 'home.pl';
use constant LOGIN_URL        => BASE_URL . 'login.pl';
use constant CHECK_URL        => BASE_URL . 'check.pl';
use constant EDIT_ACCOUNT_URL => BASE_URL . 'edit_account.pl';
use constant LOGOUT_URL       => BASE_URL . 'logout.pl';


sub new {
    my $class = shift(@_);
    my $self = bless({ @_ }, $class);

    $self->{UserAgent} = LWP::UserAgent->new;
    $self->{cookie} = HTTP::Cookies->new;
    $self->{session_id} ||= '';

    if ($self->{session_id}) {
        $self->_update_session_cookie();
    }

    return $self;
}

sub _update_session_cookie {
    my ($self) = @_;

    if (defined $self->{session_id} && $self->{session_id}) {
        $self->{cookie}->set_cookie(
                            0,
                            'BF_SESSION',
                            $self->{session_id},
                            '/',
                            DOMAIN_NAME);
        $self->{UserAgent}->cookie_jar($self->{cookie});
    }
}

# login
#  success: return session id
#  failure: return undef
sub login {
    my ($self, $email, $password) = @_;
    my $content = sprintf("next_url=%s&email=%s&password=%s",
                            HOME_URL, $email, $password);
    my $req = HTTP::Request->new(POST => LOGIN_URL, undef, $content);
    my $res = $self->{UserAgent}->request($req);

    if ($res->is_success) {
        if ($res->header('set-cookie') =~ /^BF_SESSION\=(\w+);/) {
            $self->{session_id} = $1;
            $self->_update_session_cookie();
        }
        else {
            return undef;
        }
    }
    else {
        return undef;
    }

    return $self->{session_id};
}


sub check {
    my ($self, $next_url) = (@_, HOME_URL);
    my $req = HTTP::Request->new( GET => CHECK_URL, undef, "n=$next_url" );
    my $res = $self->{UserAgent}->request($req);

    print $req->as_string,"\n";

    if ($res->is_success) {

    }
    else {
        return undef;
    }
}

sub logout {
    my ($self) = @_;

    my $req = HTTP::Request->new( GET => LOGOUT_URL );
    my $res = $self->{UserAgent}->request($req);
    
    return ($res->is_success) ? 1 : 0 ;
}

sub edit_account {
    my ($self,
        $submit, $email1, $email2, $password1, $password2,
        $diary_host, $diary_level, $diary_url, $rss_url,
        $newsletter, $info_access, @blocks) = @_;

    unless ($email1 || $email2) {
        croak "You must set email address.";
    }

    if ($password1) {
        if ($password1 ne $password2) {
            croak "bad confirmpassword";
        }
    }

    $submit      ||= 'main';
    $email1      ||= '';
    $email2      ||= '';
    $password1   ||= '';
    $password2   ||= '';
    $diary_host  ||= 8;   # 8=hatena
    $diary_level ||= 4;   # 2=friend, 3=friend's friend, 4=public
    $newsletter  ||= 'n'; # newslatter: y, biweekly, n
    $info_access ||= '';

    # TODO hash arguments and pack.
    my $content = '';
    $content .= "submit=$submit";
    $content .= "&email1=$email1&email2=$email2";
    $content .= "&password1=$password1&password2=$password2";
    $content .= "&diary_host=$diary_host&diary_level=$diary_level";
    $content .= "&diary_url=$diary_url&rss_url=$rss_url";
    $content .= "&newsletter=$newsletter&info_access=$info_access";

    for (1..6) {
        $content .= sprintf("&block%d=%s",
                            $_, ($_ < $#blocks) ? $blocks[$_-1] : '');
    }
    
    my $req = HTTP::Request->new(POST => EDIT_ACCOUNT_URL, undef, $content);
    my $res = $self->{UserAgent}->request($req);

    
    # XXX this does not check success/failure
    return ($res->is_success) ? 1 : 0 ;
}


1;

__END__
